#client.AdriftStartScreen.py

import sys

import pygame
from pygame.locals import *

import gui
import defaultStyle

from engine.Gameworld import Gameworld

from engine.AtlanSpriteGroup import AtlanSpriteGroup

from common.Utility import *

from Boat import Boat

from Cloud import Cloud

import random

import math

class AdriftStartScreen(Gameworld):
	"""
		Screen the player sees when they launch the game.
		Contains a menu to go to the credits screen, load screen, or exit.
		Has a ship in it for the player to learn the controls.
	"""

	def __init__(self):
		self.sprites = AtlanSpriteGroup()
		self.clouds = AtlanSpriteGroup()
		
		defaultStyle.init(gui)
		
		#create the GUI
		self.app = gui.Desktop()
		
		self.labelStyleCopy = gui.defaultLabelStyle.copy()
		self.labelStyleCopy['border-width'] = 0
		self.labelStyleCopy['wordwrap'] = True
		self.labelStyleCopy['autosize'] = False
		
		self.startButton = gui.Button(position = (300, 500), size = (200,0), parent = self.app, text = "PLAY")
		
		self.startButton.onClick = self.handleStart
		
		self.creditsButton = gui.Button(position = (300, 525), size=(200, 0), parent=self.app, text="ABOUT")
		self.creditsButton.onClick = self.handleCredits
		
		self.exitButton = gui.Button(position = (300, 550), size = (200,0), parent = self.app, text = "EXIT")
		
		self.exitButton.onClick = self.handleExit
		
		self.backgroundImage = getImage("oceanBackground")
		self.foregroundImage = getImage("title")
		self.okToBlowImage = getImage("okSign")
		self.okSignBounds = Rect(745, 0, 50, 50)
		self.backgroundBounds = Rect(0, 0, 800, 600)
		self.foregroundBounds = Rect(200, 100, 400, 200)
		
		#add a boat for the player to control by wind
		b = Boat(Rect(300, 300, 50, 50), None)
		self.sprites.add(b)
		
		#add some clouds
		for i in xrange(10):
			c = Cloud(Rect(random.randint(0, 800), random.randint(0, 600), 25, 25))
			self.clouds.add(c)
		
		self.clickPoint = None
		self.releasePoint = None
		self.windStrength = 2
		self.windAngle = 0
		
		self.drawArrowTimer = 0
		self.drawArrowTimerMax = 5000
		self.drawArrow = False
		
		self.drawLine = False
		
		self.currentMousePosition = None
		
		#play the ocean noises
		self.waveSound = pygame.mixer.Sound(load_sound("ocean.ogg"))
		self.waveSound.play(loops=-1)
		
		
	def handleStart(self, button):
		"""
			Go to the load screen.
		"""
		self.waveSound.stop()
		self.controller.changeScene(1)
		
	def handleCredits(self, button):
		"""
			Show the credits screen.
		"""
		self.controller.changeScene(3)
		
	def handleExit(self, button):
		"""
			Exit the game.  Triggered when the exit button is clicked.
		"""
		sys.exit()
		
	def tick(self, time):
		"""
			Update the gui.
		"""
		#update time until next wind allowed
		if self.drawArrow:
			self.drawArrowTimer += time
			if self.drawArrowTimer >= self.drawArrowTimerMax:
				self.drawArrow = False
				self.drawArrowTimer = 0
		#update logic of boat
		for sprite in self.sprites:
			sprite.setWind(self.windStrength, self.windAngle)
			sprite.update(time)
		#update logic of clouds
		for sprite in self.clouds:
			sprite.setWind(self.windStrength * 2, self.windAngle)
			sprite.update(time)
		self.app.update()
		
	def draw(self, surface):
		"""
			Draw the Adrift start screen.
			Draw background, then sprites, then clouds, then gui, 
			then the arrow for the wind
			then draw the adrift title
		"""
		surface.blit(self.backgroundImage, self.backgroundBounds)
		self.sprites.draw(surface)
		self.clouds.draw(surface)
		self.app.draw()
		if self.drawLine:
			pygame.draw.line(surface, (0, 0, 0), self.clickPoint, self.currentMousePosition, 3)
		#draw ok sign if it is ok to change wind direction
		if not self.drawArrow:
			surface.blit(self.okToBlowImage, self.okSignBounds)
		surface.blit(self.foregroundImage, self.foregroundBounds)
		
	def handleEvent(self, event):
		"""
			Handles any events generated by the user.
		"""
		#quit when the user clicks the "X"
		if event.type == pygame.QUIT:
			sys.exit()
		elif event.type == pygame.MOUSEMOTION:
			self.handleMouseMoved(event.pos)
		elif event.type == pygame.MOUSEBUTTONDOWN:
			if event.button == 1:
				self.handleMouseDown(event.pos)
		elif event.type == pygame.MOUSEBUTTONUP:
			if event.button == 1:
				self.handleMouseUp(event.pos)
				
	def handleMouseMoved(self, pos):
		"""
			Set the current pouse position.
		"""
		self.currentMousePosition = pos
				
	def handleMouseDown(self, pos):
		"""
			If the wind can change, set the new start point and start
			drawing the line
		"""
		if not self.drawArrow:
			self.clickPoint = pos
			self.drawLine = True
		
	def handleMouseUp(self, pos):
		"""
			If the wind can change, set the new end point.
			Change thee wind accordingly
		"""
		if not self.drawArrow:
			self.releasePoint = pos
			self.setWind(self.clickPoint, self.releasePoint)
			self.drawLine = False
			self.drawArrow = True
		
	def setWind(self, start, end):
		"""
			Wind vector = difference between start and end points
			of mouse click and release.
		"""
		distance = self.calculateDistance(start, end)
		strength = (distance / 800) * 3 + 1
		angle = self.calculateAngle(start, end)
		self.windStrength = strength
		self.windAngle = angle
		
	def calculateAngle(self, point1, point2):
		"""
			Calculate's the angle between a line connecting point 1 and point 2 and a line going through point 1 parallel to the x axis.
		"""
		inRads = math.atan2(point2[1] - point1[1], point2[0] - point1[0])
		return -1 * inRads * 57.295
		
	def calculateDistance(self, point1, point2):
		"""
			Calculate the absolute value of the distance between point 1 and point 2.
		"""
		return math.sqrt(((point1[0] - point2[0]) ** 2) +  ((point1[1] - point2[1]) ** 2))
		
		