#client.Boat
#I'm on a boat!

import PixelPerfect

from pygame import Rect

from common.Utility import *

import random

import math

from engine.AtlanSprite import AtlanSprite


class Boat(AtlanSprite):
	"""
		Hero class.
		Floats along, guided by the wind that the player sets.
		This class does most of the real collision detection.
	"""
	
	def __init__(self, myBounds, m):
		"""
			Create a Boat at the rectangle specified.
			m = reference to AdriftGameScreen
			m can also be none if on the start screen
		"""
		self.master = m
		AtlanSprite.__init__(self)
		self.bounds = myBounds
		self.images = getImages("boat")
		self.image = self.images[0]
		#images degrade once rotated, so keep an original copy
		self.original = self.image.copy()
		self.clock = 0
		self.flowCounter = 0
		self.windStrength = 0
		self.windDirection = 0
		self.trueX = self.bounds.x / 1.0
		self.trueY = self.bounds.y / 1.0
		self.imageBounds = self.bounds
		#boats have different behavior if they are on the
		#start screen or on the game screen
		self.SCROLL = 1
		self.WRAP = 2
		self.dead = False
		self.mode = self.WRAP
		#see PixelPerfect
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		
	def calculateCameraCoords(self, rect):
		"""
			Set the local screen coordinates.
		"""
		self.camX = self.imageBounds.x - rect.x
		self.camY = self.imageBounds.y - rect.y
		
	def flow(self, speed, angle):
		"""
			Move with the given sped and angle.
			Do not go outside the world bounds.
			Master is None, you are in the start screen.
			In that case going outside the bouonds is ok.
		"""
		radians = float(angle) / 57.295
		xDisp = math.cos(radians) * speed
		yDisp = math.sin(radians) * speed
		
		#move x and y independently.
		#if on start screen, move no matter what
		if self.master == None:
			self.trueX += xDisp
			self.bounds.x = int(self.trueX)
			self.trueY -= yDisp
			self.bounds.y = int(self.trueY)
		else:
			#if on game screen, do some weird checks to
			#make sure don't run off the screen
			#all of the checks should prevent getting stuck
			#if going the opposite direction of a wall, allow movement
			#if going the same way, do a closer check
			#I was getting some bugs before i added the additional checks
			#where you could get stuck
			#It seems to be fixed now.
			if (self.trueX + xDisp > 0 and self.trueX + xDisp + self.bounds.width < self.master.worldWidth):
				self.trueX += xDisp
				self.bounds.x = int(self.trueX)
			elif xDisp > 0 and self.trueX < 400:
				self.trueX += xDisp
				self.bounds.x = int(self.trueX)
			elif xDisp < 0 and self.trueX > 400:
				self.trueX += xDisp
				self.bounds.x = int(self.trueX)
			if (self.trueY - yDisp > 0 and self.trueY + self.bounds.height - yDisp < self.master.worldHeight):
				self.trueY -= yDisp
				self.bounds.y = int(self.trueY)
			elif yDisp < 0 and self.trueY < 400:
				self.trueY -= yDisp
				self.bounds.y = int(self.trueY)
			elif yDisp > 0 and self.trueY > 400:
				self.trueY -= yDisp
				self.bounds.y = int(self.trueY)
		
		
	def draw(self, surface):
		"""
			Draw to the screen.
			If in scroll mode (game screen), then draw to local coordinates.
			If in wrap mode (start screen), draw direct to screen)
		"""
		if self.mode == self.SCROLL:
			surface.blit(self.image, pygame.Rect(self.camX, self.camY, self.imageBounds.width, self.imageBounds.height))
				
		else:
			surface.blit(self.image, self.imageBounds)
			
	def update(self, delta):
		"""
			Update the logic of the ship.
			delta = time difference (in ms) since last frame
		"""
		self.flowCounter += delta
		if self.flowCounter > 0:
			self.flowCounter = 0
			self.flow(self.windStrength, self.windDirection)
			self.rotate(self.windDirection)
			
		#wrap mode, start screen
		#if go outside the screen, put it in the opposite side
		if self.mode == self.WRAP:
			if (self.bounds.x + self.bounds.width) > 800 + self.bounds.width:
				self.trueX = -1 * self.bounds.width
			if (self.bounds.y + self.bounds.height) > 600 + self.bounds.height:
				self.trueY = -1 * self.bounds.height
			if self.bounds.y < -1 * self.bounds.height:
				self.trueY = 600
			if self.bounds.x < -1 * self.bounds.width:
				self.trueX = 800
		#scroll mode, game screen
		#update AdriftGameScreen to our position
		#scroll will move the screen to center it around this object
		elif self.mode == self.SCROLL:
			radians = float(self.windDirection) / 57.295
			xDisp = math.cos(radians) * self.windStrength
			yDisp = math.sin(radians) * self.windStrength
			self.master.scroll(self.imageBounds, max(abs(xDisp), abs(yDisp)))
				
	def rotate(self, degrees):
		"""
			Rotate image to match wind.
			Degrees = from x axis.
			We want it to be from the y axis.
		"""
		self.image = pygame.transform.rotate(self.original, degrees)
		oldCenter = self.bounds.center
		self.bounds = self.image.get_rect();
		self.imageBounds = self.image.get_rect()
		self.bounds.center = oldCenter
		self.imageBounds.center = oldCenter
		#self.bounds.inflate_ip((-1) * self.bounds.width / 2, (-1) * self.bounds.height / 2)
		self.hitmask = pygame.surfarray.array_alpha(self.image)
		self.bounds.center = oldCenter
		
	def setWind(self, magnitude, direction):
		"""
			Set the direction and magnitude of travel based on wind.
		"""
		self.windStrength = magnitude
		self.windDirection = direction
		
	def checkCollision(self, others):
		"""
			Handle collision detection.
			Check to see if our bounds collides with anything else.
			If it does, then check pixel perfect collision detection.
			Don't handle collisions with ourself.
			Also, if the object is a doubloon instead of sinking tell the master that you found it.
			If the object was a goal, then tell the master that you found that.
			Otherwise, you hit an obstacle so sink.
		"""
		for dude in others:
			if not dude.isBoat() and not dude.isDoubloon() and not dude.isGoal():
				if dude.bounds.colliderect(self.bounds):
					if PixelPerfect._pixelPerfectCollisionDetection(self, dude):
						self.master.sunk()
			elif dude.isDoubloon():
				if dude.bounds.colliderect(self.bounds):
					if PixelPerfect._pixelPerfectCollisionDetection(self, dude):
						dude.dead = True
						self.master.acquiredDoubloon()
			elif dude.isGoal():
				if dude.bounds.colliderect(self.bounds):
					if PixelPerfect._pixelPerfectCollisionDetection(self, dude):
						self.master.victory()
		
	def isBoat(self):
		"""
			We are indeed a boat.
		"""
		return True